@@ -1,5 +1,6 @@
 /*
  * Copyright 2009 Henri Verbeet for CodeWeavers
+ * Copyright 2015-2017 Józef Kucia for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -159,7 +160,11 @@ static UINT STDMETHODCALLTYPE d3d11_query_GetDataSize(ID3D11Query *iface)
 
 static void STDMETHODCALLTYPE d3d11_query_GetDesc(ID3D11Query *iface, D3D11_QUERY_DESC *desc)
 {
-    FIXME("iface %p, desc %p stub!\n", iface, desc);
+    struct d3d_query *query = impl_from_ID3D11Query(iface);
+
+    TRACE("iface %p, desc %p.\n", iface, desc);
+
+    *desc = query->desc;
 }
 
 static const struct ID3D11QueryVtbl d3d11_query_vtbl =
@@ -315,6 +320,8 @@ static void STDMETHODCALLTYPE d3d10_query_End(ID3D10Query *iface)
 static HRESULT STDMETHODCALLTYPE d3d10_query_GetData(ID3D10Query *iface, void *data, UINT data_size, UINT flags)
 {
     struct d3d_query *query = impl_from_ID3D10Query(iface);
+    D3D11_QUERY_DATA_PIPELINE_STATISTICS d3d11_data;
+    void *d3d10_data_pointer = NULL;
     unsigned int wined3d_flags;
     HRESULT hr;
 
@@ -323,6 +330,17 @@ static HRESULT STDMETHODCALLTYPE d3d10_query_GetData(ID3D10Query *iface, void *d
     if (!data && data_size)
         return E_INVALIDARG;
 
+    if (query->desc.Query == D3D11_QUERY_PIPELINE_STATISTICS
+            && data_size == sizeof(D3D10_QUERY_DATA_PIPELINE_STATISTICS))
+    {
+        data_size = sizeof(D3D11_QUERY_DATA_PIPELINE_STATISTICS);
+        if (data)
+        {
+            d3d10_data_pointer = data;
+            data = &d3d11_data;
+        }
+    }
+
     wined3d_flags = wined3d_getdata_flags_from_d3d11_async_getdata_flags(flags);
 
     wined3d_mutex_lock();
@@ -339,6 +357,9 @@ static HRESULT STDMETHODCALLTYPE d3d10_query_GetData(ID3D10Query *iface, void *d
     }
     wined3d_mutex_unlock();
 
+    if (d3d10_data_pointer && hr == S_OK)
+        memcpy(d3d10_data_pointer, data, sizeof(D3D10_QUERY_DATA_PIPELINE_STATISTICS));
+
     return hr;
 }
 
@@ -353,6 +374,9 @@ static UINT STDMETHODCALLTYPE d3d10_query_GetDataSize(ID3D10Query *iface)
     data_size = wined3d_query_get_data_size(query->wined3d_query);
     wined3d_mutex_unlock();
 
+    if (query->desc.Query == D3D11_QUERY_PIPELINE_STATISTICS)
+        data_size = sizeof(D3D10_QUERY_DATA_PIPELINE_STATISTICS);
+
     return data_size;
 }
 
@@ -360,7 +384,11 @@ static UINT STDMETHODCALLTYPE d3d10_query_GetDataSize(ID3D10Query *iface)
 
 static void STDMETHODCALLTYPE d3d10_query_GetDesc(ID3D10Query *iface, D3D10_QUERY_DESC *desc)
 {
-    FIXME("iface %p, desc %p stub!\n", iface, desc);
+    struct d3d_query *query = impl_from_ID3D10Query(iface);
+
+    TRACE("iface %p, desc %p.\n", iface, desc);
+
+    memcpy(desc, &query->desc, sizeof(*desc));
 }
 
 static const struct ID3D10QueryVtbl d3d10_query_vtbl =
@@ -428,6 +456,9 @@ static HRESULT d3d_query_init(struct d3d_query *query, struct d3d_device *device
     query->ID3D11Query_iface.lpVtbl = &d3d11_query_vtbl;
     query->ID3D10Query_iface.lpVtbl = &d3d10_query_vtbl;
     query->refcount = 1;
+
+    query->desc = *desc;
+
     wined3d_mutex_lock();
     wined3d_private_store_init(&query->private_store);
 
