@@ -280,62 +280,78 @@ struct d3dx_param_eval
 };
 
 struct d3dx_shared_data;
+struct d3dx_top_level_parameter;
 
 struct d3dx_parameter
 {
     char magic_string[4];
+    struct d3dx_top_level_parameter *top_level_param;
+    struct d3dx_param_eval *param_eval;
     char *name;
-    char *semantic;
     void *data;
     D3DXPARAMETER_CLASS class;
     D3DXPARAMETER_TYPE  type;
     UINT rows;
     UINT columns;
     UINT element_count;
-    UINT annotation_count;
     UINT member_count;
     DWORD flags;
     UINT bytes;
     DWORD object_id;
+
+    struct d3dx_parameter *members;
+    char *semantic;
+};
+
+struct d3dx_top_level_parameter
+{
+    struct d3dx_parameter param;
+    UINT annotation_count;
+    struct d3dx_parameter *annotations;
     ULONG64 update_version;
     ULONG64 *version_counter;
-
-    struct d3dx_parameter *annotations;
-    struct d3dx_parameter *members;
-
-    struct d3dx_param_eval *param_eval;
-
-    struct d3dx_parameter *top_level_param;
-    union
-    {
-        struct d3dx_parameter *referenced_param;
-        struct d3dx_shared_data *shared_data;
-    } u;
+    struct d3dx_shared_data *shared_data;
 };
 
 struct d3dx_shared_data
 {
     void *data;
-    struct d3dx_parameter **parameters;
+    struct d3dx_top_level_parameter **parameters;
     unsigned int size, count;
     ULONG64 update_version;
 };
 
 struct d3dx9_base_effect;
 
+static inline BOOL is_top_level_parameter(struct d3dx_parameter *param)
+{
+    return &param->top_level_param->param == param;
+}
+
+static inline struct d3dx_top_level_parameter
+        *top_level_parameter_from_parameter(struct d3dx_parameter *param)
+{
+    return CONTAINING_RECORD(param, struct d3dx_top_level_parameter, param);
+}
+
 static inline ULONG64 next_update_version(ULONG64 *version_counter)
 {
     return ++*version_counter;
 }
 
-static inline BOOL is_param_dirty(struct d3dx_parameter *param, ULONG64 update_version)
+static inline BOOL is_top_level_param_dirty(struct d3dx_top_level_parameter *param, ULONG64 update_version)
 {
     struct d3dx_shared_data *shared_data;
 
-    if ((shared_data = param->top_level_param->u.shared_data))
+    if ((shared_data = param->shared_data))
         return update_version < shared_data->update_version;
     else
-        return update_version < param->top_level_param->update_version;
+        return update_version < param->update_version;
+}
+
+static inline BOOL is_param_dirty(struct d3dx_parameter *param, ULONG64 update_version)
+{
+    return is_top_level_param_dirty(param->top_level_param, update_version);
 }
 
 struct d3dx_parameter *get_parameter_by_name(struct d3dx9_base_effect *base,
@@ -345,9 +361,10 @@ struct d3dx_parameter *get_parameter_by_name(struct d3dx9_base_effect *base,
         : device->lpVtbl->method(device, args))
 #define SET_D3D_STATE(base_effect, args...) SET_D3D_STATE_(base_effect->manager, base_effect->device, args)
 
-void d3dx_create_param_eval(struct d3dx9_base_effect *base_effect, void *byte_code,
+HRESULT d3dx_create_param_eval(struct d3dx9_base_effect *base_effect, void *byte_code,
         unsigned int byte_code_size, D3DXPARAMETER_TYPE type,
-        struct d3dx_param_eval **peval, ULONG64 *version_counter) DECLSPEC_HIDDEN;
+        struct d3dx_param_eval **peval, ULONG64 *version_counter,
+        const char **skip_constants, unsigned int skip_constants_count) DECLSPEC_HIDDEN;
 void d3dx_free_param_eval(struct d3dx_param_eval *peval) DECLSPEC_HIDDEN;
 HRESULT d3dx_evaluate_parameter(struct d3dx_param_eval *peval,
         const struct d3dx_parameter *param, void *param_value) DECLSPEC_HIDDEN;
@@ -355,4 +372,13 @@ HRESULT d3dx_param_eval_set_shader_constants(ID3DXEffectStateManager *manager, s
         struct d3dx_param_eval *peval, BOOL update_all) DECLSPEC_HIDDEN;
 BOOL is_param_eval_input_dirty(struct d3dx_param_eval *peval, ULONG64 update_version) DECLSPEC_HIDDEN;
 
+struct ctab_constant {
+    D3DXCONSTANT_DESC desc;
+    WORD constantinfo_reserved;
+    struct ctab_constant *constants;
+};
+
+const struct ctab_constant *d3dx_shader_get_ctab_constant(ID3DXConstantTable *iface,
+        D3DXHANDLE constant) DECLSPEC_HIDDEN;
+
 #endif /* __WINE_D3DX9_PRIVATE_H */
