@@ -674,7 +674,6 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     HKEY hkey;
     LPWSTR username, companyname;
     SYSTEM_INFO sys_info;
-    SYSTEMTIME systemtime;
     LANGID langid;
 
     static const WCHAR szCommonFilesFolder[] = {'C','o','m','m','o','n','F','i','l','e','s','F','o','l','d','e','r',0};
@@ -736,8 +735,6 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     };
     static const WCHAR szUSERNAME[] = {'U','S','E','R','N','A','M','E',0};
     static const WCHAR szCOMPANYNAME[] = {'C','O','M','P','A','N','Y','N','A','M','E',0};
-    static const WCHAR szDate[] = {'D','a','t','e',0};
-    static const WCHAR szTime[] = {'T','i','m','e',0};
     static const WCHAR szUserLanguageID[] = {'U','s','e','r','L','a','n','g','u','a','g','e','I','D',0};
     static const WCHAR szSystemLangID[] = {'S','y','s','t','e','m','L','a','n','g','u','a','g','e','I','D',0};
     static const WCHAR szProductState[] = {'P','r','o','d','u','c','t','S','t','a','t','e',0};
@@ -966,22 +963,6 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     if ( set_user_sid_prop( package ) != ERROR_SUCCESS)
         ERR("Failed to set the UserSID property\n");
 
-    /* Date and time properties */
-    GetSystemTime( &systemtime );
-    if (GetDateFormatW( LOCALE_USER_DEFAULT, DATE_SHORTDATE, &systemtime,
-                        NULL, bufstr, sizeof(bufstr)/sizeof(bufstr[0]) ))
-        msi_set_property( package->db, szDate, bufstr, -1 );
-    else
-        ERR("Couldn't set Date property: GetDateFormat failed with error %d\n", GetLastError());
-
-    if (GetTimeFormatW( LOCALE_USER_DEFAULT,
-                        TIME_FORCE24HOURFORMAT | TIME_NOTIMEMARKER,
-                        &systemtime, NULL, bufstr,
-                        sizeof(bufstr)/sizeof(bufstr[0]) ))
-        msi_set_property( package->db, szTime, bufstr, -1 );
-    else
-        ERR("Couldn't set Time property: GetTimeFormat failed with error %d\n", GetLastError());
-
     set_msi_assembly_prop( package );
 
     langid = GetUserDefaultLangID();
@@ -1478,11 +1459,7 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
     MSISUMMARYINFO *si;
     BOOL delete_on_close = FALSE;
     LPWSTR productname;
-    static const WCHAR date_format[] =
-        {'M','/','d','/','y','y','y','y',0};
-    static const WCHAR time_format[] =
-        {'H','H','\'',':','\'','m','m','\'',':','\'','s','s',0};
-    WCHAR timet[100], datet[100], info_template[1024], info_message[1024];
+    WCHAR *info_template;
 
     TRACE("%s %p\n", debugstr_w(szPackage), pPackage);
 
@@ -1635,11 +1612,9 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
 	msiobj_release(&data_row->hdr);
 	return ERROR_OUTOFMEMORY;
     }
-    GetTimeFormatW(LOCALE_USER_DEFAULT, 0, NULL, time_format, timet, 100);
-    GetDateFormatW(LOCALE_USER_DEFAULT, 0, NULL, date_format, datet, 100);
-    LoadStringW(msi_hInstance, IDS_INFO_LOGGINGSTART, info_template, 1024);
-    sprintfW(info_message, info_template, datet, timet);
-    MSI_RecordSetStringW(info_row, 0, info_message);
+    info_template = msi_get_error_message(package->db, MSIERR_INFO_LOGGINGSTART);
+    MSI_RecordSetStringW(info_row, 0, info_template);
+    msi_free(info_template);
     MSI_ProcessMessage(package, INSTALLMESSAGE_INFO|MB_ICONHAND, info_row);
 
     MSI_ProcessMessage(package, INSTALLMESSAGE_COMMONDATA, data_row);
@@ -1792,9 +1767,9 @@ static INT internal_ui_handler(MSIPACKAGE *package, INSTALLMESSAGE eMessageType,
     case INSTALLMESSAGE_SHOWDIALOG:
     {
         LPWSTR dialog = msi_dup_record_field(record, 0);
-        UINT rc = ACTION_DialogBox(package, dialog);
+        INT rc = ACTION_DialogBox(package, dialog);
         msi_free(dialog);
-        return (rc == ERROR_SUCCESS);
+        return rc;
     }
     case INSTALLMESSAGE_ACTIONSTART:
     {
@@ -1843,11 +1818,69 @@ static INT internal_ui_handler(MSIPACKAGE *package, INSTALLMESSAGE eMessageType,
             ;
         }
     default:
-        FIXME("internal UI not implemented for message 0x%08x (UI level = %x)", eMessageType, package->ui_level);
+        FIXME("internal UI not implemented for message 0x%08x (UI level = %x)\n", eMessageType, package->ui_level);
         return 0;
     }
 }
 
+static const WCHAR szActionNotFound[] = {'D','E','B','U','G',':',' ','E','r','r','o','r',' ','[','1',']',':',' ',' ','A','c','t','i','o','n',' ','n','o','t',' ','f','o','u','n','d',':',' ','[','2',']',0};
+
+static const struct
+{
+    int id;
+    const WCHAR *text;
+}
+internal_errors[] =
+{
+    {2726, szActionNotFound},
+    {0}
+};
+
+static LPCWSTR get_internal_error_message(int error)
+{
+    int i = 0;
+
+    while (internal_errors[i].id != 0)
+    {
+        if (internal_errors[i].id == error)
+            return internal_errors[i].text;
+        i++;
+    }
+
+    FIXME("missing error message %d\n", error);
+    return NULL;
+}
+
+/* Returned string must be freed */
+LPWSTR msi_get_error_message(MSIDATABASE *db, int error)
+{
+    static const WCHAR query[] =
+        {'S','E','L','E','C','T',' ','`','M','e','s','s','a','g','e','`',' ',
+         'F','R','O','M',' ','`','E','r','r','o','r','`',' ','W','H','E','R','E',' ',
+         '`','E','r','r','o','r','`',' ','=',' ','%','i',0};
+    MSIRECORD *record;
+    LPWSTR ret = NULL;
+
+    if ((record = MSI_QueryGetRecord(db, query, error)))
+    {
+        ret = msi_dup_record_field(record, 1);
+        msiobj_release(&record->hdr);
+    }
+    else if (error < 2000)
+    {
+        int len = LoadStringW(msi_hInstance, IDS_ERROR_BASE + error, (LPWSTR) &ret, 0);
+        if (len)
+        {
+            ret = msi_alloc((len + 1) * sizeof(WCHAR));
+            LoadStringW(msi_hInstance, IDS_ERROR_BASE + error, ret, len + 1);
+        }
+        else
+            ret = NULL;
+    }
+
+    return ret;
+}
+
 INT MSI_ProcessMessageVerbatim(MSIPACKAGE *package, INSTALLMESSAGE eMessageType, MSIRECORD *record)
 {
     LPWSTR message = {0};
@@ -1954,34 +1987,49 @@ INT MSI_ProcessMessage( MSIPACKAGE *package, INSTALLMESSAGE eMessageType, MSIREC
     case INSTALLMESSAGE_OUTOFDISKSPACE:
         if (MSI_RecordGetInteger(record, 1) != MSI_NULL_INTEGER)
         {
-            WCHAR template_prefix[1024];
-            WCHAR *template_rec, *template;
-            UINT prefix_id = 0;
+            /* error message */
+
+            LPWSTR template;
+            LPWSTR template_rec = NULL, template_prefix = NULL;
+            int error = MSI_RecordGetInteger(record, 1);
 
             if (MSI_RecordIsNull(record, 0))
             {
-                LoadStringW(msi_hInstance, IDS_INSTALLERROR, template_prefix, 1024);
-                MSI_RecordSetStringW(record, 0, template_prefix);
-                break;
+                if (error >= 32)
+                {
+                    template_rec = msi_get_error_message(package->db, error);
+
+                    if (!template_rec && error >= 2000)
+                    {
+                        /* internal error, not localized */
+                        if ((template_rec = (LPWSTR) get_internal_error_message(error)))
+                        {
+                            MSI_RecordSetStringW(record, 0, template_rec);
+                            MSI_ProcessMessageVerbatim(package, INSTALLMESSAGE_INFO, record);
+                        }
+                        template_rec = msi_get_error_message(package->db, MSIERR_INSTALLERROR);
+                        MSI_RecordSetStringW(record, 0, template_rec);
+                        MSI_ProcessMessageVerbatim(package, eMessageType, record);
+                        msi_free(template_rec);
+                        return 0;
+                    }
+                }
             }
+            else
+                template_rec = msi_dup_record_field(record, 0);
 
-            if ((eMessageType & 0xff000000) == INSTALLMESSAGE_USER)
-                break;
+            template_prefix = msi_get_error_message(package->db, eMessageType >> 24);
+            if (!template_prefix) template_prefix = strdupW(szEmpty);
 
-            switch(eMessageType & 0xff000000)
+            if (!template_rec)
             {
-            case INSTALLMESSAGE_FATALEXIT: prefix_id = IDS_FATALEXIT; break;
-            case INSTALLMESSAGE_ERROR: prefix_id = IDS_ERROR; break;
-            case INSTALLMESSAGE_WARNING: prefix_id = IDS_WARNING; break;
-            case INSTALLMESSAGE_INFO: prefix_id = IDS_INFO; break;
-            case INSTALLMESSAGE_OUTOFDISKSPACE: prefix_id = IDS_OUTOFDISKSPACE; break;
+                /* always returns 0 */
+                MSI_RecordSetStringW(record, 0, template_prefix);
+                MSI_ProcessMessageVerbatim(package, eMessageType, record);
+                msi_free(template_prefix);
+                return 0;
             }
 
-            LoadStringW(msi_hInstance, prefix_id, template_prefix, 1024);
-
-            template_rec = msi_dup_record_field(record, 0);
-            if (!template_rec) return ERROR_OUTOFMEMORY;
-
             template = msi_alloc((strlenW(template_rec) + strlenW(template_prefix) + 1) * sizeof(WCHAR));
             if (!template) return ERROR_OUTOFMEMORY;
 
@@ -1989,21 +2037,16 @@ INT MSI_ProcessMessage( MSIPACKAGE *package, INSTALLMESSAGE eMessageType, MSIREC
             strcatW(template, template_rec);
             MSI_RecordSetStringW(record, 0, template);
 
+            msi_free(template_prefix);
             msi_free(template_rec);
             msi_free(template);
         }
         break;
     case INSTALLMESSAGE_ACTIONSTART:
     {
-        WCHAR template_s[1024];
-        static const WCHAR time_format[] =
-            {'H','H','\'',':','\'','m','m','\'',':','\'','s','s',0};
-        WCHAR timet[100], template[1024];
-
-        LoadStringW(msi_hInstance, IDS_ACTIONSTART, template_s, 1024);
-        GetTimeFormatW(LOCALE_USER_DEFAULT, 0, NULL, time_format, timet, 100);
-        sprintfW(template, template_s, timet);
+        WCHAR *template = msi_get_error_message(package->db, MSIERR_ACTIONSTART);
         MSI_RecordSetStringW(record, 0, template);
+        msi_free(template);
 
         msi_free(package->LastAction);
         msi_free(package->LastActionTemplate);
@@ -2028,9 +2071,9 @@ INT MSI_ProcessMessage( MSIPACKAGE *package, INSTALLMESSAGE eMessageType, MSIREC
         break;
     case INSTALLMESSAGE_COMMONDATA:
     {
-        WCHAR template[1024];
-        LoadStringW(msi_hInstance, IDS_COMMONDATA, template, 1024);
+        WCHAR *template = msi_get_error_message(package->db, MSIERR_COMMONDATA);
         MSI_RecordSetStringW(record, 0, template);
+        msi_free(template);
     }
     break;
     }
@@ -2258,9 +2301,45 @@ static MSIRECORD *msi_get_property_row( MSIDATABASE *db, LPCWSTR name )
     MSIQUERY *view;
     UINT r;
 
+    static const WCHAR szDate[] = {'D','a','t','e',0};
+    static const WCHAR szTime[] = {'T','i','m','e',0};
+    WCHAR *buffer;
+    int length;
+
     if (!name || !*name)
         return NULL;
 
+    if (!strcmpW(name, szDate))
+    {
+        length = GetDateFormatW(LOCALE_USER_DEFAULT, DATE_SHORTDATE, NULL, NULL, NULL, 0);
+        if (!length)
+            return NULL;
+        buffer = msi_alloc(length * sizeof(WCHAR));
+        GetDateFormatW(LOCALE_USER_DEFAULT, DATE_SHORTDATE, NULL, NULL, buffer, sizeof(WCHAR));
+
+        row = MSI_CreateRecord(1);
+        if (!row)
+            return NULL;
+        MSI_RecordSetStringW(row, 1, buffer);
+        msi_free(buffer);
+        return row;
+    }
+    else if (!strcmpW(name, szTime))
+    {
+        length = GetTimeFormatW(LOCALE_USER_DEFAULT, TIME_NOTIMEMARKER, NULL, NULL, NULL, 0);
+        if (!length)
+            return NULL;
+        buffer = msi_alloc(length * sizeof(WCHAR));
+        GetTimeFormatW(LOCALE_USER_DEFAULT, TIME_NOTIMEMARKER, NULL, NULL, buffer, sizeof(WCHAR));
+
+        row = MSI_CreateRecord(1);
+        if (!row)
+            return NULL;
+        MSI_RecordSetStringW(row, 1, buffer);
+        msi_free(buffer);
+        return row;
+    }
+
     rec = MSI_CreateRecord(1);
     if (!rec)
         return NULL;
