@@ -42,6 +42,11 @@ struct iosb
     void         *out_data;     /* output data */
 };
 
+struct async_queue
+{
+    struct list queue;          /* queue of async objects */
+};
+
 /* operations valid on file descriptor objects */
 struct fd_ops
 {
@@ -52,13 +57,13 @@ struct fd_ops
     /* get file information */
     enum server_fd_type (*get_fd_type)(struct fd *fd);
     /* perform a read on the file */
-    obj_handle_t (*read)(struct fd *, struct async *, file_pos_t );
+    int (*read)(struct fd *, struct async *, file_pos_t );
     /* perform a write on the file */
-    obj_handle_t (*write)(struct fd *, struct async *, file_pos_t );
+    int (*write)(struct fd *, struct async *, file_pos_t );
     /* flush the object buffers */
-    obj_handle_t (*flush)(struct fd *, struct async *);
+    int (*flush)(struct fd *, struct async *);
     /* perform an ioctl on the file */
-    obj_handle_t (*ioctl)(struct fd *fd, ioctl_code_t code, struct async *async );
+    int (*ioctl)(struct fd *fd, ioctl_code_t code, struct async *async );
     /* queue an async operation */
     void (*queue_async)(struct fd *, struct async *async, int type, int count);
     /* selected events for async i/o need an update */
@@ -97,14 +102,14 @@ extern int default_fd_signaled( struct object *obj, struct wait_queue_entry *ent
 extern unsigned int default_fd_map_access( struct object *obj, unsigned int access );
 extern int default_fd_get_poll_events( struct fd *fd );
 extern void default_poll_event( struct fd *fd, int event );
-extern int fd_queue_async( struct fd *fd, struct async *async, int type );
+extern void fd_queue_async( struct fd *fd, struct async *async, int type );
 extern void fd_async_wake_up( struct fd *fd, int type, unsigned int status );
 extern void fd_reselect_async( struct fd *fd, struct async_queue *queue );
-extern obj_handle_t no_fd_read( struct fd *fd, struct async *async, file_pos_t pos );
-extern obj_handle_t no_fd_write( struct fd *fd, struct async *async, file_pos_t pos );
-extern obj_handle_t no_fd_flush( struct fd *fd, struct async *async );
-extern obj_handle_t no_fd_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
-extern obj_handle_t default_fd_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
+extern int no_fd_read( struct fd *fd, struct async *async, file_pos_t pos );
+extern int no_fd_write( struct fd *fd, struct async *async, file_pos_t pos );
+extern int no_fd_flush( struct fd *fd, struct async *async );
+extern int no_fd_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
+extern int default_fd_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
 extern void no_fd_queue_async( struct fd *fd, struct async *async, int type, int count );
 extern void default_fd_queue_async( struct fd *fd, struct async *async, int type, int count );
 extern void default_fd_reselect_async( struct fd *fd, struct async_queue *queue );
@@ -173,14 +178,13 @@ extern int is_serial_fd( struct fd *fd );
 extern struct object *create_serial( struct fd *fd );
 
 /* async I/O functions */
-extern struct async_queue *create_async_queue( struct fd *fd );
 extern void free_async_queue( struct async_queue *queue );
-extern struct async *create_async( struct thread *thread, const async_data_t *data, struct iosb *iosb );
-extern struct async *create_request_async( struct thread *thread, const async_data_t *data );
+extern struct async *create_async( struct fd *fd, struct thread *thread, const async_data_t *data, struct iosb *iosb );
+extern struct async *create_request_async( struct fd *fd, const async_data_t *data );
+extern obj_handle_t async_handoff( struct async *async, int success, data_size_t *result );
 extern void queue_async( struct async_queue *queue, struct async *async );
 extern void async_set_timeout( struct async *async, timeout_t timeout, unsigned int status );
 extern void async_set_result( struct object *obj, unsigned int status, apc_param_t total );
-extern int async_queued( struct async_queue *queue );
 extern int async_waiting( struct async_queue *queue );
 extern void async_terminate( struct async *async, unsigned int status );
 extern void async_wake_up( struct async_queue *queue, unsigned int status );
@@ -192,6 +196,17 @@ extern int async_is_blocking( struct async *async );
 extern struct async *find_pending_async( struct async_queue *queue );
 extern void cancel_process_asyncs( struct process *process );
 
+static inline void init_async_queue( struct async_queue *queue )
+{
+    list_init( &queue->queue );
+}
+
+static inline int async_queued( struct async_queue *queue )
+{
+    return !list_empty( &queue->queue );
+}
+
+
 /* access rights that require Unix read permission */
 #define FILE_UNIX_READ_ACCESS (FILE_READ_DATA|FILE_READ_ATTRIBUTES|FILE_READ_EA)
 
